import {
  select,
  csv,
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  format,
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');
/*  
    d.acceleration = +d.acceleration;
    d.cylinders = +d.cylinders;
    d.displacement = +d.displacement;
    d.horsepower = +d.horsepower;
    d['model year'] = +d['model year'];
    d.mpg = +d.mpg;
    d.weight = +d.weight;
*/

const render = (data) => {
  // value accessors:
  const title = 'Cars : Horsepower vs MPG';
  const xValue = (d) => d.mpg;
  const xAxisLabel = 'mpg';
  const yValue = (d) => d.horsepower;
  const yAxisLabel = 'Horsepower';

  const circleRadius = 10;
  // adding margins:
  const margin = {
    top: 60,
    right: 40,
    bottom: 77,
    left: 150,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  // setting up linear and band scales:
  const xScale = scaleLinear() // we can envoke scale linear as a function to create an instance of a d3 scale
    // .domain([min(data, xValue), max(data, xValue)]) // is the same as the extent func
    .domain(extent(data, xValue))
    .range([0, innerWidth]) // this means the maximum population should map to the maximum bar width
    .nice(); // make it so it ends on a tick

  // change from  scale point to scale linear
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]) // allows the data elements to be arranged from top to bottom
    .nice();
  // lets append our rectangles to g instead of svg so that we have a margin...
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );
  // since billions for the SI standard is G..
  //but we want it to say B so lets create a format function...
  const xAxisTickFormat = (number) =>
    format('.3s')(number).replace('G', 'B');

  // setting up axes:
  const xAxis = axisBottom(xScale)
    .tickFormat(xAxisTickFormat)
    .tickSize(-innerHeight) // create vertical lines...
    .tickPadding(17);

  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(15);

  const yAxisG = g.append('g').call(yAxis);

  yAxisG
    .selectAll('.domain') // get the domain class from this group tag, get the tick class, and then get its  nested line element
    .remove(); // remove it.. ( to remove the y axis line)
  // select all starts at the class level... <g class ="domain"...>  or <g class = tick...>

  // making y axis label
  yAxisG
    .append('text')
    .attr('class', 'x-axis-label')
    .attr('fill', 'black') // make the text appear
    .attr('y', -90)
    .attr('x', -innerHeight / 2)
    .attr('text-anchor', 'middle')
    .attr('transform', `rotate(-90)`) // instead of translate lets
    .text(yAxisLabel);

  const xAxisG = g // made xAxisG so that we can access this group
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);

  xAxisG
    .select('.domain') // get the domain class from this group tag
    .remove();

  // making x axis label
  xAxisG
    .append('text')
    .attr('class', 'x-axis-label')
    .attr('fill', 'black') // make the text appear
    .attr('y', 60)
    .attr('x', innerWidth / 2)
    .text(xAxisLabel);

  // change rect to circles for scatter plot
  g.selectAll('circle')
    .data(data) // d3 selects all existing rectangles ( this is the d3 data join)
    .enter()
    .append('circle')
    .attr('cy', (d) => yScale(yValue(d))) // changed from bandscale to point scale
    // .attr('cy',(d) => yScale(yValue(d)) + yScale.bandwidth() / 2)// manually move the circle centers down
    .attr('cx', (d) => xScale(xValue(d)))
    // .attr('r', yScale.bandwidth() / 2); // /2 makes the circles not overlap
    .attr('r', circleRadius);

  // add title
  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
    .text(title);
};

// loading and parsing data:
let csv_url =
  'https://raw.githubusercontent.com/WhteRabb-t/CP341-datavis/main/auto-mpg.csv';
csv(csv_url).then((data) => {
  /*  use this to see what needs to be parsed as a number
0{ acceleration: "12",
car name: "chevrolet chevelle malibu",
cylinders: "8",
displacement: "307",
horsepower: "130",
model year: "70",
mpg: "18",
origin: "1",
weight: "3504",
}
   */
  // part that parses and modifies the data:
  data.forEach((d) => {
    d.acceleration = +d.acceleration;
    d.cylinders = +d.cylinders;
    d.displacement = +d.displacement;
    d.horsepower = +d.horsepower;
    d['model year'] = +d['model year'];
    d.mpg = +d.mpg;
    d.weight = +d.weight;
  });

  // render the data after parsing it
  render(data);
});

//  csv() is a promis that resolves when this data is loaded
//  this csv() function of D3 makes an http request ,
//  loads the csv string from data.csv
//  then parses the csv into an array of objects

//keys of each object represent the columns
// values represent the values in the table
